#include "sjog.h"
#include "sjog_list.h"
#include "sjog_list_callbacks.h"
#include "sjog_enlightenment.h"

#include <libintl.h>
#define _(String) gettext (String)

/* -------------------------------------------------------------------------- */

/*
 * Call the configuration file parsing and creates the list window
 * (only the window, not the list).
 */
void
sjog_list_init()
{
   /* call the parsing of the config file */
   if (eMenu)
      sjog_init_tree_entries();
   else
      sjog_init_list_entries();

   /* create the list window and register callbacks */
   listWindow = windowsAreTransient ?
      gtk_window_new(GTK_WINDOW_POPUP) : gtk_window_new(GTK_WINDOW_TOPLEVEL);

   gtk_window_set_title(GTK_WINDOW(listWindow), "S-Jog");
   gtk_signal_connect(GTK_OBJECT(listWindow), "delete-event",
                      GTK_SIGNAL_FUNC(sjog_list_signal_delete_event_callback),
                      NULL);

   /* TT - This callback is used to determine where to move the window
    * just before it's shown, and just after the widget has been allocated.
    */
   gtk_signal_connect(GTK_OBJECT(listWindow), "realize",
                      GTK_SIGNAL_FUNC(sjog_list_signal_realize_callback),
                      NULL);
   
   /* creates a 1 column width list */
   cmdListUI = gtk_clist_new(1);
   gtk_clist_set_column_auto_resize(GTK_CLIST(cmdListUI), 0, TRUE);

   /* be warned when a row is selected */
   gtk_signal_connect(GTK_OBJECT(cmdListUI), "select-row",
                      GTK_SIGNAL_FUNC(sjog_list_row_selected_callback), NULL);

   gtk_container_add(GTK_CONTAINER(listWindow), cmdListUI);

}

/* -------------------------------------------------------------------------- */

/*
 * Builds the widgets of the actions list.
 * @arg list the list model.
 */
void
sjog_list_set_list()
{
   guint length, i = 0;      /* commands list length and loop index */
   sjog_cmd_t *cmd;          /* temporary pointer for commands insertion loop */
   gchar *item[1];           /* GtkCList needs an array of pointers to gchar */


   /* default is to select 1st item */
   selectedIndex = 0;

   /* get the number of items to put in the list */
   length = g_list_length(list);

   if (length == 0)
   {
      fprintf(stderr, _("Error: the list contains no items, exiting\n"));
      exit(2);
   }

   D("list contains %d items\n", length);

   /* loops over the items and add them to the list */
   for (i = 0; i <= length - 1; i++)
   {
      cmd = (sjog_cmd_t *) g_list_nth_data(list, i);
      item[0] = cmd->name;
      gtk_clist_append(GTK_CLIST(cmdListUI), item);
   }

   /* select the 1st item */
   gtk_clist_select_row(GTK_CLIST(cmdListUI), selectedIndex, 0);

}

/* -------------------------------------------------------------------------- */

/*
 * Shows the list window.
 */
void
sjog_list_show()
{

   D_ENTER;

   if (listWindow == NULL)
   {
      sjog_list_init();
      if (eMenu)
         sjog_tree_set_list();
      else
         sjog_list_set_list();
   }

   if (resetMenu == TRUE)
   {
      /* select the 1st item */
      gtk_clist_select_row(GTK_CLIST(cmdListUI), 0, 0);
   }

   if (!GTK_WIDGET_VISIBLE(listWindow))
   {
      /* TT - The above sjog_window_set_position() function is now depracated,
       * as the functionaliy has been moved into the "realize" event callback
       */
      gtk_widget_show_all(listWindow);
   }
}

/* -------------------------------------------------------------------------- */

/*
 * Hides the list window.
 */
void
sjog_list_hide()
{
   D_ENTER;

   gtk_widget_hide_all(listWindow);
}

/* -------------------------------------------------------------------------- */

/*
 * Called when the jog is pushed while we are in LIST_MODE. Either executes
 * a command or jump to one of the other modes (VOLUME, BRIGHTNESS..)
 */
sjog_mode_t *
sjog_list_exec_cmd()
{
   sjog_cmd_t *sjog_cmd;               /* sjog_cmd to execute */
   static sjog_cmd_t *prev_cmd = NULL; /* previously executed command */
   GList *list_element;                /* glib element that contains the cmd */
   GNode *tree_element;                /* glib element that contains the cmd */
   gchar *cmd;                         /* cmd string to execute */

   /* remove the timer and hide the window */
   sjog_list_hide();

   D_ENTER;

   if (eMenu)
   {
      tree_element = g_node_nth_child(currentTreeNode, selectedIndex);
      sjog_cmd = (sjog_cmd_t *) tree_element->data;
   }
   else
   {
      list_element = g_list_nth(list, selectedIndex);
      sjog_cmd = (sjog_cmd_t *) list_element->data;
   }

   D("sjog_cmd->path = %s\n", sjog_cmd->path);

   /* redo allows execution of the last executed command  */
   if (strcmp(sjog_cmd->path, "redo") == 0) {

      /* no previous command */
      if (prev_cmd == NULL) return;

      sjog_cmd = prev_cmd;
   }
   else {
      prev_cmd = sjog_cmd;
   }

   /* submenu handling */
   if (eMenu && sjog_cmd->type == SJOG_CMD_TYPE_MENU)
   {
      /* return to parent menu if special "parent" item is selected */
      if (strcmp(sjog_cmd->path, "parent") == 0)
      {
         sjog_list_go_back();
      }

      /* enter a new menu only if there is something in it */
      else if (g_node_n_children(tree_element))
      {
         gtk_clist_clear(GTK_CLIST(cmdListUI));
         currentTreeNode = tree_element;
         sjog_tree_set_list();
      }

      return listMode;
   }
   else {
      sjog_mode_t *next_mode = sjog_exec_cmd(sjog_cmd->path);
      return next_mode == NULL ? scrollMode : next_mode;
   }

   return NULL;
}

/* -------------------------------------------------------------------------- */

void
sjog_list_go_back()
{
   guint index;

   if (eMenu)
   {
      D_ENTER;

      if (currentTreeNode->parent)
      {
         /* force selected index */
         index = g_node_child_position(currentTreeNode->parent, currentTreeNode);
         gtk_clist_clear(GTK_CLIST(cmdListUI));
         currentTreeNode = currentTreeNode->parent;
         sjog_tree_set_list();

         /* force re-display */
         sjog_list_hide();
         sjog_list_show();

         gtk_clist_select_row(GTK_CLIST(cmdListUI), index, 0);
      }
      else
         sjog_list_hide();
   }
}

/*
 * Selects the next item in the application list.
 */
void
sjog_list_select_next()
{
   guint length;

   if (eMenu)
      length = g_node_n_children(currentTreeNode);
   else
      length = g_list_length(list);

   /* if we hit bottom, go to top */
   if (selectedIndex == (length - 1))
      selectedIndex = 0;
   else
      selectedIndex++;

   D("selectedIndex=%d\n", selectedIndex);

   gtk_clist_select_row(GTK_CLIST(cmdListUI), selectedIndex, 0);
   //sjog_list_reset_timer();

}

/* -------------------------------------------------------------------------- */

/*
 * Selects the previous item in the application list.
 */
void
sjog_list_select_previous()
{
   guint length;

   if (eMenu)
      length = g_node_n_children(currentTreeNode);
   else
      length = g_list_length(list);

   /* if we hit top, go to bottom */
   if (selectedIndex == 0)
      selectedIndex = length - 1;
   else
      selectedIndex--;

   D("selectedIndex=%d\n", selectedIndex);

   gtk_clist_select_row(GTK_CLIST(cmdListUI), selectedIndex, 0);
}

void
sjog_list_mode_init(sjog_mode_t *list_mode)
{
   list_mode->bottom_cb = sjog_list_select_next;
   list_mode->pushed_cb = sjog_list_exec_cmd;
   list_mode->top_cb = sjog_list_select_previous;
   list_mode->start = sjog_list_show;
   list_mode->stop = sjog_list_hide;
   list_mode->needsTimeout = TRUE;
}

/*
 * modelines
 * ex:set ai ts=3 sw=3:
 * ei:set itab=spaces noat:
 * vim:set et:
 */
