#include <sys/ioctl.h>
#include <linux/soundcard.h>

#include "sjog.h"
#include "sjog_volume.h"
#include "sjog_volume_callbacks.h"

GtkWidget *volumeWindow;     /* volume dialog box */
GtkWidget *progressBar;      /* widget to show volume level */
guint currentVolume;         /* startup volume */
guint mixerFD = 0;           /* mixer file descriptor */

#include <libintl.h>
#define _(String) gettext (String)

/* -------------------------------------------------------------------------- */

/* 
 * Called by sjog_jog_position_changed when we are in SJOG_MODE_VOLUME.
 * Checks the jog and increase/decrease the volume level according to
 * its position. Also polls the actual volume to update the progress bar
 * in case it has been modified by another program.
 */
void
sjog_volume_bottom()
{
   D_ENTER;

   /* update currentVolume just in case it has been modified externally */
   currentVolume = sjog_get_volume();

   if (currentVolume > 0)
   {
     /* round the volume before decrease */
     currentVolume = (int) (((float) currentVolume / 10) + 0.5) * 10;
     currentVolume -= 10;
     sjog_set_volume(currentVolume);
   }

   gtk_progress_set_value(GTK_PROGRESS(progressBar), currentVolume);
}

sjog_mode_t *
sjog_volume_pushed()
{
   D_ENTER;

   return scrollMode;
}

void
sjog_volume_top()
{
   D_ENTER;

   /* update currentVolume just in case it has been modified externally */
   currentVolume = sjog_get_volume();

   if (currentVolume < 100)
   {
      /* round the volume before increase */
      currentVolume = (int) (((float) currentVolume / 10) + 0.5) * 10;
      currentVolume += 10;
      sjog_set_volume(currentVolume);
   }

   gtk_progress_set_value(GTK_PROGRESS(progressBar), currentVolume);
}

/* -------------------------------------------------------------------------- */

void
sjog_set_volume(gint volume)
{

   gint vol = 0;

   D("volume=%d\n", volume);

   vol = (volume << 8) + volume;
   ioctl(mixerFD, MIXER_WRITE(SOUND_MIXER_VOLUME), &vol);

   /* finally sets the progress bar to the new volume value */
   gtk_progress_set_value(GTK_PROGRESS(progressBar), currentVolume);

}

/* -------------------------------------------------------------------------- */

gint
sjog_get_volume()
{

   gint r, l, vol;

   D_ENTER;

   ioctl(mixerFD, MIXER_READ(SOUND_MIXER_VOLUME), &vol);

   l = vol & 0xff;
   r = (vol & 0xff00) >> 8;

   return (r + l) / 2;

}

/* -------------------------------------------------------------------------- */

void
sjog_init_mixer_fd()
{
   gboolean success = FALSE;

   /*
      TT - First, if mixerDevice is set, try to open that device. Otherwise,
      try to open the /dev/mixer device, and finally the /dev/sound/mixer.
    */

   if (mixerDevice != NULL)
   {
      if ((mixerFD = open(mixerDevice, O_RDWR)) == -1)
      {
         fprintf(stderr, _("Warning: Unable to open mixer device %s: %s\n"),
                 mixerDevice, strerror(errno));
      }
      else
         success = TRUE;
   }

   if ((!success) && ((mixerFD = open(MIXER_DEVICE, O_RDWR)) == -1))
   {
      fprintf(stderr, _("Warning: Unable to open mixer device %s: %s\n"),
              MIXER_DEVICE, strerror(errno));
      fprintf(stderr, _("Trying %s... "), MIXER_DEVICE_DEVFS);
      if ((mixerFD = open(MIXER_DEVICE_DEVFS, O_RDWR)) == -1)
      {
         fprintf(stderr, _("\nWarning: Unable to open mixer device %s: %s\n"),
                 MIXER_DEVICE_DEVFS, strerror(errno));
         fprintf(stderr, _("Giving up on opening a mixer...\n"));
      }
      else
      {
         success = TRUE;
         fprintf(stderr, _("success\n"));
      }
   }
   else
      success = TRUE;
}

/* -------------------------------------------------------------------------- */

/* Shows the screen volume level setting dialog.
 */
void
sjog_volume_show()
{

   /* first, try to open the mixer device */
   if (mixerFD == 0)
   {
      sjog_init_mixer_fd();

      /* do nothing if we can't open it */
      if (mixerFD == 0)
      {
         fprintf(stderr, "mixer initialization failed\n");
         sjog_set_mode(SJOG_MODE_LIST);
         return;
      }
   }

   /* first launch needs gui initialization */
   if (volumeWindow == NULL)
      init_volume_gui();

   if (!GTK_WIDGET_VISIBLE(volumeWindow))
   {
      /* read the volume value from mixer and update the progress bar */
      currentVolume = sjog_get_volume();
      gtk_progress_set_value(GTK_PROGRESS(progressBar), currentVolume);

      /* popup the volume window */
      gtk_widget_show_all(volumeWindow);
   }
}

/* -------------------------------------------------------------------------- */

/* 
 * Hides the volume level setting dialog.
 */
void
sjog_volume_hide()
{
   D_ENTER;

   if (mixerFD != 0)
   {
      close(mixerFD);
      mixerFD = 0;
   }
   gtk_widget_hide_all(volumeWindow);
}

/* 
 * Builds the widgets of the volume dialog window
 * Widgets hierarchy:
 *      volumeWindow -> vbox -> volume_pixmap
 *                               -> progressBar
 */
void
init_volume_gui()
{

   GtkAdjustment *adjust;
   GtkWidget *vbox;
   GdkPixmap *gd_pixmap;
   GtkWidget *volume_pixmap;

   /* vertical box */
   vbox = gtk_vbox_new(FALSE, 10);

   /* volume window */
   volumeWindow = windowsAreTransient ?
      gtk_window_new(GTK_WINDOW_POPUP) : gtk_window_new(GTK_WINDOW_TOPLEVEL);

   gtk_container_set_border_width(GTK_CONTAINER(volumeWindow), 10);
   gtk_window_set_position(GTK_WINDOW(volumeWindow), GTK_WIN_POS_CENTER);
   gtk_window_set_title(GTK_WINDOW(volumeWindow), _("S-Jog: Volume"));
   gtk_signal_connect(GTK_OBJECT(volumeWindow), "delete-event",
                      GTK_SIGNAL_FUNC
                      (sjog_volume_signal_delete_event_callback), NULL);
   gtk_container_add(GTK_CONTAINER(volumeWindow), vbox);

   /* volume image */
   gtk_widget_realize(volumeWindow);
   gd_pixmap =
      gdk_pixmap_create_from_xpm(gtk_widget_get_parent_window(vbox), NULL,
                                 NULL,
                                 strdup(PKGDATADIR
                                        "/pixmaps/sjog-volume.xpm"));

   volume_pixmap = gtk_pixmap_new(gd_pixmap, NULL);
   gtk_container_add(GTK_CONTAINER(vbox), volume_pixmap);

   /* volume progress bar */
   adjust =
      (GtkAdjustment *) gtk_adjustment_new(DEFAULT_VALUE, MIN_VALUE,
                                           MAX_VALUE, STEP_INC, PAGE_INC,
                                           PAGE_SIZE);
   progressBar = gtk_progress_bar_new_with_adjustment(adjust);
   gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(progressBar),
                                    GTK_PROGRESS_LEFT_TO_RIGHT);
   gtk_progress_bar_set_bar_style(GTK_PROGRESS_BAR(progressBar),
                                  GTK_PROGRESS_CONTINUOUS);
   gtk_progress_set_format_string(GTK_PROGRESS(progressBar), "%v%%");
   gtk_progress_set_show_text(GTK_PROGRESS(progressBar), TRUE);

   gtk_container_add(GTK_CONTAINER(vbox), progressBar);
}

void
sjog_volume_mode_init(sjog_mode_t *volume_mode)
{
   D_ENTER;

   volume_mode->bottom_cb = sjog_volume_bottom;
   volume_mode->pushed_cb = sjog_volume_pushed;
   volume_mode->top_cb = sjog_volume_top;
   volume_mode->start = sjog_volume_show;
   volume_mode->stop = sjog_volume_hide;
   volume_mode->needsTimeout = TRUE;
}


/*
 * modelines
 * ex:set ai ts=3 sw=3:
 * ei:set itab=spaces noat:
 * vim:set et:
 */
